# -*- coding: utf-8 -*-
"""
Created on Fri Sep 13 04:07:07 2019

@author: Taufiq
"""
import requests

def test_checkUrl1():
    url1 = "https://rawgit.com/WCRP-CMIP/CMIP6_CVs/master/src/CMIP6_source_id.html"
    R = requests.get(url1)
    R.close()
    assert R.status_code == 200
    
def test_checkUrl2():
    url2 = "https://rawgit.com/WCRP-CMIP/CMIP6_CVs/master/src/CMIP6_experiment_id.html"
    R = requests.get(url2)
    R.close()
    assert R.status_code == 200
    
def test_checkUrl3():
    url3_4 = "https://esgf-data.dkrz.de/search/cmip6-dkrz/"
    url3_3 = "https://esgf-index1.ceda.ac.uk/search/cmip6-ceda/"
    url3_1 = "https://esgf-node.llnl.gov/search/cmip6/"
    url3_2 = "https://esgf-node.ipsl.upmc.fr/search/cmip6-ipsl/"
    try:
        if (requests.get(url3_1,timeout=10)):
            R=requests.get(url3_1)
            R.close()
    except:
        try:
            if (requests.get(url3_2,timeout=10)):
                R=requests.get(url3_2)
                R.close()
        except:
            try:
                if (requests.get(url3_3,timeout=10)):
                    R=requests.get(url3_3)
                    R.close()
            except:
                try:
                    if (requests.get(url3_4,timeout=10)):
                        R=requests.get(url3_4)
                        R.close()
                except:
                    print("\nAll servers down!!\nCheck back later.")
        
    assert R.status_code == 200
    
def test_checkUrl4():
    url4_1 = "https://esgf-node.llnl.gov/esg-search/wget?project=CMIP6"
    url4_2 = "https://esgf-node.ipsl.upmc.fr/esg-search/wget?project=CMIP6"
    url4_3 = "https://esgf-index1.ceda.ac.uk/esg-search/wget?project=CMIP6"
    url4_4 = "https://esgf-data.dkrz.de/esg-search/wget?project=CMIP6"
    try:
        if (requests.get(url4_1,timeout=10)):
            R=requests.get(url4_1)
            R.close()
    except:
        try:
            if (requests.get(url4_2,timeout=10)):
                R=requests.get(url4_2)
                R.close()
        except:
            try:
                if (requests.get(url4_3,timeout=10)):
                    R=requests.get(url4_3)
                    R.close()
            except:
                try:
                    if (requests.get(url4_4,timeout=10)):
                        R=requests.get(url4_4)
                        R.close()
                except:
                    print("\nAll servers down!!\nCheck back later.")
        
    assert R.status_code == 200


# -*- coding: utf-8 -*-
"""
Created on Tue Sep  3 18:39:39 2019
"""
import re
import pandas as pd # pandas for data handling
import pkg_resources

from acccmip6.utilities.util import _fetch_url, _choose_server

class CMIP6DB:
    
    _Turl = "https://rawgit.com/WCRP-CMIP/CMIP6_CVs/master/src/CMIP6_source_id.html"
    _ETurl = "https://rawgit.com/WCRP-CMIP/CMIP6_CVs/master/src/CMIP6_experiment_id.html"
    _Curl = "https://esgf-node.llnl.gov/search/cmip6/"
    
    def __init__(self, **options):
        self._total = 0
        self._avail = 0
        self._holder = []
        self._fdata = []
    
    @classmethod
    def _set_curl(cls, url):
        cls._curl = url
        return cls._curl
        
    def available_models(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_source_id_',self._fdata))
                print("\nCurrently ", self._avail," models has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_source_id_')[zz+2].split('" name="')[0])
                print("Available models: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_source_id_',self._fdata))
                print("\nCurrently ", self._avail," models has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_source_id_')[zz+2].split('" name="')[0])
                print("Available models: \n\n")
                return self._holder
    
    def all_models(self):
        with _fetch_url(self._set_curl(self._Turl)) as self._fdata:
            self._total = len(re.findall('<tr><td>',self._fdata))
            print("\nCMIP6 has ", self._total," models in total!\n")
            for zz in range(self._total):
                self._holder.append(self._fdata.split('<tr><td>')[zz+1].split('</td>\\n')[0])
            print("List of all CMIP6 models: \n\n")
            return self._holder
            
    def available_experiments(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_experiment_id_',self._fdata))
                print("\nCurrently ", self._avail," experiments has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_experiment_id_')[zz+2].split('" name="')[0])
                print("Available experiments: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_experiment_id_',self._fdata))
                print("\nCurrently ", self._avail," experiments has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_experiment_id_')[zz+2].split('" name="')[0])
                print("Available experiments: \n\n")
                return self._holder
    
    def all_experiments(self):
        with _fetch_url(self._set_curl(self._ETurl)) as self._fdata:
            self._total = len(re.findall('<tr><td>',self._fdata))
            print("\nCMIP6 has ", self._total," experiments in total!\n")
            for zz in range(self._total):
                self._holder.append(self._fdata.split('<tr><td>')[zz+1].split('</td>\\n')[0])
            print("List of all CMIP6 experiments: \n\n")
            return self._holder
            
    def CMIP6_variables(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_variable_id_',self._fdata))
                print("\nCurrently ", self._avail," variables has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_variable_id_')[zz+2].split('" name="')[0])
                print("Available variables: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_variable_id_',self._fdata))
                print("\nCurrently ", self._avail," variables has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_variable_id_')[zz+2].split('" name="')[0])
                print("Available variables: \n\n")
                return self._holder
        
    def available_frequencies(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_frequency_',self._fdata))
                print("\nCurrently ", self._avail," frequencies has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_frequency_')[zz+2].split('" name="')[0])
                print("Available frequencies: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_frequency_',self._fdata))
                print("\nCurrently ", self._avail," frequencies has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_frequency_')[zz+2].split('" name="')[0])
                print("Available frequencies: \n\n")
                return self._holder
        
    def available_realmns(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_realm_',self._fdata))
                print("\nCurrently ", self._avail," realms has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_realm_')[zz+2].split('" name="')[0])
                print("Available realms: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_realm_',self._fdata))
                print("\nCurrently ", self._avail," realms has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_realm_')[zz+2].split('" name="')[0])
                print("Available frequencies: \n\n")
                return self._holder
            
    def var_stdName(self):
        try:
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_cf_standard_name_',self._fdata))
                print("\nCurrently ", self._avail," variables has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_cf_standard_name_')[zz+2].split('" name="')[0])
                print("Available variables: \n\n")
                return self._holder
        except:
            self._Curl=_choose_server()
            with _fetch_url(self._set_curl(self._Curl)) as self._fdata:
                self._avail = len(re.findall('id="checkbox_cf_standard_name_',self._fdata))
                print("\nCurrently ", self._avail," variables has outputs!\n")
                for zz in range(self._avail):
                    self._holder.append(self._fdata.split('checkbox_cf_standard_name_')[zz+2].split('" name="')[0])
                print("Available frequencies: \n\n")
                return self._holder
    
    @staticmethod
    def _get_definition(exp):
        resource_package = __name__
        resource_path = '/'.join(('data', 'CMIP6_exps.xlsx'))
        tmp = pkg_resources.resource_stream(resource_package, resource_path)
        did=pd.read_excel(tmp)
        exp_name=pd.DataFrame(did,columns=['canonical_name'])
        exp_def=pd.DataFrame(did,columns=['description'])
        for zz in range(len(exp_name.values)):
            if (exp_name.values[zz]==exp):
                definition=(exp_def.values[zz][0]).split('\n')[0].strip()
        return definition
    
    @staticmethod
    def _get_longName(var):
        resource_package = __name__
        resource_path = '/'.join(('data', 'var_list.xlsx'))
        tmp = pkg_resources.resource_stream(resource_package, resource_path)
        did=pd.read_excel(tmp)
        v=pd.DataFrame(did,columns=['variable'])
        v_long=pd.DataFrame(did,columns=['Long_name'])
        for zz in range(len(v.values)):
            if (v.values[zz]==var):
                longName=v_long.values[zz][0]
        return longName
            


# -*- coding: utf-8 -*-
"""
Created on Tue Sep  3 22:16:28 2019

@author: Taufiq
"""

from acccmip6.utilities.util import _extract_info, _Construct_urls
from acccmip6.utilities.checkers import _checkers

class SearchDB(object):

    _check = 'No'
    
    def __init__(self, **kwargs):
        self._var = kwargs.get('variable', None)
        self._mod = kwargs.get('model', None)
        self._exp = kwargs.get('experiment', None)
        self._freq = kwargs.get('frequency', None)
        self._realm = kwargs.get('realm', None)
        self.n_files = kwargs.get('n_files', None)
        self.rlzn = kwargs.get('rlzn', None)
        self.year = kwargs.get('year', None)
        self.links = kwargs.get('links', None)

    @classmethod
    def _set_check(cls, val):
        cls._check = val
        return cls._check

    @property
    def model(self):
        return self._mod

    @model.setter
    def model(self, val):
        self._mod=[0]
        mods = [x.strip() for x in val.split(',')]
        for zz in range(len(mods)):
            if (self._check == 'Yes'):
                self._mod.append(_checkers(mods[zz])._check_model())
            else:
                self._mod.append(mods[zz])
        self._mod.remove(0)

    @property
    def experiment(self):
        return self._exp

    @experiment.setter
    def experiment(self, val):
        self._exp = [0]
        exps = [x.strip() for x in val.split(',')]
        for zz in range(len(exps)):
            if (self._check == 'Yes'):
                self._exp.append(_checkers(exps[zz])._check_exp())
            else:
                self._exp.append(exps[zz])
        self._exp.remove(0)
            
    @property
    def variable(self):
        return self._var

    @variable.setter
    def variable(self, val):
        self._var = [0]
        vars = [x.strip() for x in val.split(',')]
        for zz in range(len(vars)):
            if (self._check == 'Yes'):
                self._var.append(_checkers(vars[zz])._check_var())
            else:
                self._var.append(vars[zz])
        self._var.remove(0)
            
    @property
    def realm(self):
        return self._realm

    @realm.setter
    def realm(self, val):
        self._realm = [0]
        realms = [x.strip() for x in val.split(',')]
        for zz in range(len(realms)):
            if (self._check == 'Yes'):
                self._realm.append(_checkers(realms[zz])._check_realm())
            else:
                self._realm.append(realms[zz])
        self._realm.remove(0)
            
    @property
    def frequency(self):
        return self._freq

    @frequency.setter
    def frequency(self, val):
        self._freq = [0]
        freqs = [x.strip() for x in val.split(',')]
        for zz in range(len(freqs)):
            if (self._check == 'Yes'):
                self._freq.append(_checkers(freqs[zz])._check_freq())
            else:
                self._freq.append(freqs[zz])
        self._freq.remove(0)
        
    @property
    def realization(self):
        return self.rlzn

    @realization.setter
    def realization(self, val):
        self.rlzn = [0]
        rlzns = [x.strip() for x in val.split(',')]
        for zz in range(len(rlzns)):
            self.rlzn.append(rlzns[zz])
        self.rlzn.remove(0)
        
    @property
    def years(self):
        return self.year

    @years.setter
    def years(self, val):
        self.year = [0]
        year = [x.strip() for x in val.split(',')]
        for zz in range(len(year)):
            self.year.append(year[zz])
        self.year.remove(0)
    
    
    def get_links(self, manual):
        if manual==0:
            links = _Construct_urls(self._var, self._mod, self._realm, self._exp, self._freq)._get_wget(0)
        else:
            links = _Construct_urls(self._var, self._mod, self._realm, self._exp, self._freq)._get_wget(1)
        return links
    
    def get_info(self):
        info = _extract_info(self._var, self._mod, self._realm, self._exp, self._freq, self.n_files, self.rlzn, self.year,self.links)._get_info()
        return info
    
    def get_url(self):
        url = _Construct_urls(self._var, self._mod, self._realm, self._exp, self._freq)._get_url()
        return url
    


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 13 04:25:50 2019

@author: Taufiq
"""

from acccmip6.utilities.c6db import SearchDB

def setup_module(module):
    global d
    d = SearchDB()
    
def test_model_setters():
    d.model = 'model1, model2, model3, modelN'
    assert d._mod == ['model1', 'model2', 'model3', 'modelN']
    
def test_exp_setters():
    d.experiment = 'exp1, exp2, exp3, expN'
    assert d._exp == ['exp1', 'exp2', 'exp3', 'expN']
    
def test_var_setters():
    d.variable = 'var1, var2, var3, varN'
    assert d._var == ['var1', 'var2', 'var3', 'varN']
    
def test_freq_setters():
    d.frequency = 'freq1, freq2, freq3, freq4'
    assert d._freq == ['freq1', 'freq2', 'freq3', 'freq4']


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_accmip6
----------------------------------

Tests for `accmip6` module.
"""
import pytest
from pathlib import Path

from acccmip6.utilities.c6db import SearchDB
from acccmip6.utilities.util import _dir_path, _Construct_urls
    
def test_url_getter():
    d = SearchDB()
    d.variable = 'var1, var2, var3, varN'
    url = d.get_url()
    durl=_Construct_urls(['var1', 'var2', 'var3', 'varN'],None,None,None,None)._Durl
    assert url == durl+"&variable=var1&variable=var2&variable=var3&variable=varN&limit=10000"
        
def test_dir_path():
    d = _dir_path()
    p=Path('.')
    assert d._get_dir('') == p.absolute() / 'CMIP6'
    



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 13 01:13:29 2019

@author: Taufiq
"""
from pathlib import Path
import unittest

import pkg_resources


class TestRequirements(unittest.TestCase):
    def test_requirements(self):
        """Recursively confirm that requirements are available."""
        requirements = (Path(__file__).parents[1] / 'requirements.in').read_text().strip().split('\n')
        requirements = [r.strip() for r in requirements]
        requirements = [r for r in sorted(requirements) if r and not r.startswith('#')]
        for requirement in requirements:
            with self.subTest(requirement=requirement):
                pkg_resources.require(requirement)



"""A setuptools based setup module for acccmip6"""
# -*- coding: utf-8 -*-

from codecs import open
from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as readme_file:
    readme = readme_file.read()

requirements = [
    'pandas',
    'requests',
    'setuptools',
    'xlrd',
]

test_requirements = [
    'pytest',
    'requests',
]

setup(
    name='acccmip6',
    version='4.0.4',
    description="Package for accessing CMIP6 database in real-time",
    long_description=readme,
    author="Taufiq Hassan",
    author_email='taufiq.hassanmozumder@email.ucr.edu',
    url='https://github.com/TaufiqHassan/acccmip6',
    packages=find_packages(exclude=['docs', 'tests']),
    package_data = {
            'utilities':['data/*.xlsx'],
            },
    entry_points={
        'console_scripts':[
            'acccmip6=acccmip6.cli:main',
            ],
        },
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite='tests',
    tests_require=test_requirements,
)
